from compgeom.pnt2d import Pnt2D
from geometry.curves.curve import Curve
from geometry.curves.circlearc import CircleArc
import math


class Circle(Curve):

    deltaAng = math.pi / 32.0

    def __init__(self, _center=None, _circ1=None):
        super(Curve, self).__init__()
        self.type = 'CIRCLE'
        self.center = _center
        self.circ1 = _circ1
        self.nPts = 0
        self.eqPoly = []
        if _center is not None:
            self.nPts += 1
        if _circ1 is not None:
            self.nPts += 1
        self.radius = 0.0
        self.ang1 = 0.0  # Angle of circle point (0 <= angle < +2PI)

        if self.nPts == 2:
            drX1 = self.circ1.getX() - self.center.getX()
            drY1 = self.circ1.getY() - self.center.getY()
            self.radius = math.sqrt(drX1 * drX1 + drY1 * drY1)
            if self.radius > 0.0:
                # compute angle for the circle point
                self.ang1 = math.atan2(drY1, drX1)  # -PI < angle <= +PI
                if abs(self.ang1) < Curve.PARAM_TOL:
                    self.ang1 = 0.0
                if self.ang1 < 0.0:
                    self.ang1 = 2.0 * math.pi + self.ang1  # 0 <= angle < +2PI

    # ---------------------------------------------------------------------
    def computeEquivPolyline(self):
        # Compute equivalent polygonal of current circle
        self.eqPoly = []
        drX = self.circ1.getX() - self.center.getX()
        drY = self.circ1.getY() - self.center.getY()
        radius = math.sqrt(drX * drX + drY * drY)
        if radius > 0.0:
            # compute angle for the circle point
            angInit = math.atan2(drY, drX)  # -PI < angle <= +PI
            if abs(angInit) < Curve.PARAM_TOL:
                angInit = 0.0
            if angInit < 0.0:
                angInit = 2.0 * math.pi + angInit  # 0 <= angle < +2PI
            nSegs = int((2.0 * math.pi) / Circle.deltaAng)
            self.eqPoly.append(self.circ1)
            ang = angInit
            for i in range(1, nSegs):
                ang += Circle.deltaAng
                x = self.center.getX() + radius * math.cos(ang)
                y = self.center.getY() + radius * math.sin(ang)
                ptOn = Pnt2D(x, y)
                self.eqPoly.append(ptOn)
            self.eqPoly.append(self.circ1)

    # ---------------------------------------------------------------------
    def setPoints(self, _x0, _y0, _x1, _y1):
        self.center = Pnt2D(_x0, _y0)
        self.circ1 = Pnt2D(_x1, _y1)
        self.nPts = 2
        drX1 = self.circ1.getX() - self.center.getX()
        drY1 = self.circ1.getY() - self.center.getY()
        self.radius = math.sqrt(drX1 * drX1 + drY1 * drY1)
        if self.radius > 0.0:
            # compute angle for the circle point
            self.ang1 = math.atan2(drY1, drX1)  # -PI < angle <= +PI
            if abs(self.ang1) < Curve.PARAM_TOL:
                self.ang1 = 0.0
            if self.ang1 < 0.0:
                self.ang1 = 2.0 * math.pi + self.ang1  # 0 <= angle < +2PI
            self.computeEquivPolyline()

    # ---------------------------------------------------------------------
    def addCtrlPoint(self, _x, _y):
        # **** COMPLETE HERE: CIRCLE_01 ****

    # ---------------------------------------------------------------------
    def isPossible(self):
        # **** COMPLETE HERE: CIRCLE_02 ****
        return True

    # ---------------------------------------------------------------------
    def getCtrlPoints(self):
        tempPts = []
        if self.nPts == 0:
            return tempPts
        if self.nPts == 1:
            tempPts.append(self.center)
            return tempPts
        tempPts.append(self.center)
        tempPts.append(self.circ1)
        return tempPts

    # ---------------------------------------------------------------------
    def getNumberOfReshapeCtrlPoints(self):
        return 3

    # ---------------------------------------------------------------------
    def getReshapeCtrlPoints(self):
        ctrlPts = []
        ctrlPts.append(self.center)
        ctrlPts.append(self.circ1)
        # Create a fictitious control point diametrically opposed to the
        # circle control point 
        drX1 = self.circ1.getX() - self.center.getX()
        drY1 = self.circ1.getY() - self.center.getY()
        circ2 = Pnt2D(self.center.getX() - drX1, self.center.getY() - drY1)
        ctrlPts.append(circ2)
        return ctrlPts

    # ---------------------------------------------------------------------
    def setCtrlPoint(self, _id, _x, _y, _tol):
        status = False
        # **** COMPLETE HERE: CIRCLE_03 ****
        if status:
            # Recompute equivalent polygonal of current circle
            self.computeEquivPolyline()
        return status

    # ---------------------------------------------------------------------
    def isStraight(self, _tol):
        return False

    # ---------------------------------------------------------------------
    def isClosed(self):
        # **** COMPLETE HERE: CIRCLE_04 ****
        return True

    # ---------------------------------------------------------------------
    def evalPoint(self, _t):
        # **** COMPLETE HERE: CIRCLE_05 ****
        return pt

    # ---------------------------------------------------------------------
    def evalPointTangent(self, _t):
        # **** COMPLETE HERE: CIRCLE_06 ****
        return pt, tangVec

    # ---------------------------------------------------------------------
    def splitRaw(self, _t):
        # **** COMPLETE HERE: CIRCLE_07 ****
        return left, right

    # ---------------------------------------------------------------------
    def split(self, _t):
        left, right = self.splitRaw(_t) 
        if left != None:
            left.computeEquivPolyline()
        if right != None:
            right.computeEquivPolyline()
        return left, right

    # ---------------------------------------------------------------------
    def join(self, _joinCurve, _pt, _tol):
        return False, None, 'Cannot join segments:\n A closed curve may not be joined with another curve.'

    # ---------------------------------------------------------------------
    def getEquivPolyline(self):
        equivPoly = []
        if self.nPts != 2:
            return equivPoly

        # Compute equivalent polygonal of current circle if
        # it is not already created
        if self.eqPoly == []:
            self.computeEquivPolyline()

        for i in range(0, len(self.eqPoly)):
            equivPoly.append(self.eqPoly[i])
        return equivPoly

    # ---------------------------------------------------------------------
    def getEquivPolylineCollecting(self, _pt):
        # **** COMPLETE HERE: CIRCLE_08 ****

    # ---------------------------------------------------------------------
    def closestPoint(self, _x, _y):
        # **** COMPLETE HERE: CIRCLE_09 ****
        return True, clstPt, dist, t, tangVec

    # ---------------------------------------------------------------------
    def closestPointParam(self, _x, _y, _tStart):
        clstPt, tangVec = self.evalPointTangent(_tStart)
        if ((abs(clstPt.getX() - _x) < Curve.COORD_TOL) and
            (abs(clstPt.getY() - _y) < Curve.COORD_TOL)):
            return True, clstPt, 0.0, _tStart, tangVec

        status, clstPt, dist, t, tangVec = self.closestPoint(_x, _y)
        return status, clstPt, dist, t, tangVec

    # ---------------------------------------------------------------------
    def getBoundBox(self):
        # **** COMPLETE HERE: CIRCLE_10 ****
        return xmin, xmax, ymin, ymax

    # ---------------------------------------------------------------------
    def getXinit(self):
        return self.circ1.getX()

    # ---------------------------------------------------------------------
    def getYinit(self):
        return self.circ1.getY()

    # ---------------------------------------------------------------------
    def getXend(self):
        return self.circ1.getX()

    # ---------------------------------------------------------------------
    def getYend(self):
        return self.circ1.getY()

    # ---------------------------------------------------------------------
    def getPntInit(self):
        return self.circ1

    # ---------------------------------------------------------------------
    def getPntEnd(self):
        return self.circ1

    # ---------------------------------------------------------------------
    def length(self):
        L = 2.0 * math.pi * self.radius
        return L
