from compgeom.compgeom import CompGeom
from compgeom.pnt2d import Pnt2D
from geometry.curves.curve import Curve
from geometry.curves.line import Line
import math


class QuadBezier(Curve):
    def __init__(self, _pt0=None, _pt1=None, _pt2=None):
        super(Curve, self).__init__()
        self.type = 'QUADBEZIER'
        self.pt0 = _pt0
        self.pt1 = _pt1
        self.pt2 = _pt2
        self.nPts = 0
        self.eqPoly = []
        if _pt0 is not None:
            self.nPts += 1
        if _pt1 is not None:
            self.nPts += 1
        if _pt2 is not None:
            self.nPts += 1

    # ---------------------------------------------------------------------
    def computeEquivPolyline(self):
        # Compute equivalent polygonal of current quadratic Bezier
        self.eqPoly = []
        len1 = Pnt2D.euclidiandistance(self.pt0, self.pt1)
        len2 = Pnt2D.euclidiandistance(self.pt1, self.pt2)
        length = len1 + len2
        tol = length * 0.001
        self.eqPoly = Curve.genEquivPolyline(self, self.eqPoly, tol)
        self.eqPoly.append(self.pt2)

    # ---------------------------------------------------------------------
    def setPoints(self, _x0, _y0, _x1, _y1, _x2, _y2):
        self.pt0 = Pnt2D(_x0, _y0)
        self.pt1 = Pnt2D(_x1, _y1)
        self.pt2 = Pnt2D(_x2, _y2)
        self.nPts = 3
        self.computeEquivPolyline()

    # ---------------------------------------------------------------------
    def addCtrlPoint(self, _x, _y):
        # **** COMPLETE HERE: QUADBEZIER_01 ****

    # ---------------------------------------------------------------------
    def isPossible(self):
        # **** COMPLETE HERE: QUADBEZIER_02 ****
        return True

    # ---------------------------------------------------------------------
    def getCtrlPoints(self):
        tempPts = []
        if self.nPts == 0:
            return tempPts
        if self.nPts == 1:
            tempPts.append(self.pt0)
            return tempPts
        if self.nPts == 2:
            tempPts.append(self.pt0)
            tempPts.append(self.pt2)
            return tempPts
        tempPts.append(self.pt0)
        tempPts.append(self.pt1)
        tempPts.append(self.pt2)
        return tempPts

    # ---------------------------------------------------------------------
    def setCtrlPoint(self, _id, _x, _y, _tol):
        # **** COMPLETE HERE: QUADBEZIER_03 ****

        # Recompute equivalent polygonal of current quadratic Bezier
        self.computeEquivPolyline()
        return True

    # ---------------------------------------------------------------------
    def isStraight(self, _tol):
        if not CompGeom.pickLine(self.pt0, self.pt2, self.pt1, _tol):
            return False
        return True

    # ---------------------------------------------------------------------
    def isClosed(self):
        # **** COMPLETE HERE: QUADBEZIER_04 ****
        return False

    # ---------------------------------------------------------------------
    def evalPoint(self, _t):
        # **** COMPLETE HERE: QUADBEZIER_05 ****
        return pOn

    # ---------------------------------------------------------------------
    def evalPointTangent(self, _t):
        # **** COMPLETE HERE: QUADBEZIER_06 ****
        return pt, tangVec

    # ---------------------------------------------------------------------
    def splitRaw(self, _t):
        # **** COMPLETE HERE: QUADBEZIER_07 ****
        return left, right

    # ---------------------------------------------------------------------
    def split(self, _t):
        left, right = self.splitRaw(_t) 
        if left != None:
            left.computeEquivPolyline()
        if right != None:
            right.computeEquivPolyline()
        return left, right

    # ---------------------------------------------------------------------
    def join(self, _joinCurve, _pt, _tol):
        if _joinCurve.getType() != 'QUADBEZIER':
            return False, None, 'Cannot join segments:\n A quadratic Bezier curve may be joined only with another quadratic Bezier.'

        if self.isStraight(_tol) or _joinCurve.isStraight(_tol):
            return False, None, 'Cannot join segments:\n A quadratic Bezier curve to be joined is straight.'

        # Get the order of two curves.
        # It is assumed that the given point _pt is the common point of
        # the curves to be joined.
        selfPtInit = self.getPntInit()
        if Pnt2D.euclidiandistance(selfPtInit, _pt) < _tol:
            crvA = _joinCurve
            crvB = self
        else:
            crvA = self
            crvB = _joinCurve

        # Get the control points of each curve.
        crvApts = crvA.getCtrlPoints()
        crvApt0 = crvApts[0]
        crvApt1 = crvApts[1]
        crvBpts = crvB.getCtrlPoints()
        crvBpt1 = crvBpts[1]
        crvBpt2 = crvBpts[2]

        # Check for aligned central control segments.
        if not CompGeom.pickLine(crvApt1, crvBpt1, _pt, _tol):
            return False, None, 'Cannot join segments:\n Incompatible control points of quadratic Bezier curves.'

        # Compute the intersetion point of the first control segment with the last control segment.
        areaA01B1 = Pnt2D.area2d(crvApt0, crvApt1, crvBpt1)
        if abs(areaA01B1) < CompGeom.ABSTOL:
            return False, None, 'Cannot join segments:\n Incompatible control points of quadratic Bezier curves.'
        areaA01B2 = Pnt2D.area2d(crvApt0, crvApt1, crvBpt2)
        if abs(areaA01B2) < CompGeom.ABSTOL:
            return False, None, 'Cannot join segments:\n Incompatible control points of quadratic Bezier curves.'
        tB21 = areaA01B2 / (areaA01B2 - areaA01B1)
        vB21 = crvBpt1 - crvBpt2
        crvPt1 = crvBpt2 + vB21*tB21

        # Create a circle arc joining the two arcs.
        crv = QuadBezier(crvApt0, crvPt1, crvBpt2)
        crv.computeEquivPolyline()

        return True, crv, None

    # ---------------------------------------------------------------------
    def getEquivPolyline(self):
        equivPoly = []
        if self.nPts != 3:
            return equivPoly

        # Compute equivalent polygonal of current quadratic Bezier if
        # it is not already created
        if self.eqPoly == []:
            self.computeEquivPolyline()

        for i in range(0, len(self.eqPoly)):
            equivPoly.append(self.eqPoly[i])
        return equivPoly

    # ---------------------------------------------------------------------
    def getEquivPolylineCollecting(self, _pt):
        # **** COMPLETE HERE: QUADBEZIER_08 ****

    # ---------------------------------------------------------------------
    def closestPointSeg(self, _x, _y):
        # Compute equivalent polygonal of current quadratic Bezier if
        # it is not already created
        if self.eqPoly == []:
            self.computeEquivPolyline()

        aux = Line(self.eqPoly[0], self.eqPoly[1])
        status, clstPtSeg, d, t, tang = aux.closestPoint(_x, _y)
        xOn = clstPtSeg.getX()
        yOn = clstPtSeg.getY()
        dmin = d
        seg = 0

        for i in range(1, len(self.eqPoly) - 1):
            aux = Line(self.eqPoly[i], self.eqPoly[i + 1])
            status, clstPtSeg, d, t, tang = aux.closestPoint(_x, _y)
            if d < dmin:
                xOn = clstPtSeg.getX()
                yOn = clstPtSeg.getY()
                dmin = d
                seg = i

        arcLen = 0.0
        for i in range(0, seg):
            arcLen += math.sqrt((self.eqPoly[i + 1].getX() - self.eqPoly[i].getX()) *
                                (self.eqPoly[i + 1].getX() - self.eqPoly[i].getX()) +
                                (self.eqPoly[i + 1].getY() - self.eqPoly[i].getY()) *
                                (self.eqPoly[i + 1].getY() - self.eqPoly[i].getY()))
        arcLen += math.sqrt((xOn - self.eqPoly[seg].getX()) *
                            (xOn - self.eqPoly[seg].getX()) +
                            (yOn - self.eqPoly[seg].getY()) *
                            (yOn - self.eqPoly[seg].getY()))

        clstPt = Pnt2D(xOn, yOn)
        return clstPt, dmin, seg, arcLen

    # ---------------------------------------------------------------------
    def closestPoint(self, _x, _y):
        # **** COMPLETE HERE: QUADBEZIER_09 ****
        return status, clstPt, dmin, t, tang

    # ---------------------------------------------------------------------
    def closestPointParam(self, _x, _y, _tStart):
        t = _tStart
        if t < Curve.PARAM_TOL:
            t = 0.0
            seg = 0
            clstPt = self.eqPoly[seg]
            tang = self.eqPoly[seg + 1] - self.eqPoly[seg]
            status = True
        elif t > 1.0 - Curve.PARAM_TOL:
            t = 1.0
            seg = len(self.eqPoly) - 2
            clstPt = self.eqPoly[seg + 1]
            tang = self.eqPoly[seg + 1] - self.eqPoly[seg]
            status = True
        else:
            status, clstPt, t, tang = Curve.ParamCurveClosestPt(self, _x, _y, t)
        if status:
            dmin = math.sqrt((clstPt.getX() - _x) * (clstPt.getX() - _x) +
                             (clstPt.getY() - _y) * (clstPt.getY() - _y))
        else:
            dmin = 0.0
        return status, clstPt, dmin, t, tang

    # ---------------------------------------------------------------------
    def getBoundBox(self):
        # **** COMPLETE HERE: QUADBEZIER_10 ****
        return xmin, xmax, ymin, ymax

    # ---------------------------------------------------------------------
    def getXinit(self):
        return self.pt0.getX()

    # ---------------------------------------------------------------------
    def getYinit(self):
        return self.pt0.getY()

    # ---------------------------------------------------------------------
    def getXend(self):
        return self.pt2.getX()

    # ---------------------------------------------------------------------
    def getYend(self):
        return self.pt2.getY()

    # ---------------------------------------------------------------------
    def getPntInit(self):
        return self.pt0

    # ---------------------------------------------------------------------
    def getPntEnd(self):
        return self.pt2

    # ---------------------------------------------------------------------
    def length(self):
        if self.nPts < 3:
            return 0.0
        L = 0.0
        for i in range(0, len(self.eqPoly) - 1):
            L += math.sqrt((self.eqPoly[i + 1].getX() - self.eqPoly[i].getX()) *
                           (self.eqPoly[i + 1].getX() - self.eqPoly[i].getX()) +
                           (self.eqPoly[i + 1].getY() - self.eqPoly[i].getY()) *
                           (self.eqPoly[i + 1].getY() - self.eqPoly[i].getY()))
        return L
